<?php
/**
Plugin Name: Zoom Search
Plugin URI: http://www.wrensoft.com/zoom/support/tutorial_wordpress.html
Description: Easily incorporate Zoom Search, a fast custom website search engine, using pre-indexed data files into your WordPress site.
Version: 0.1.7000
Author: Wrensoft
Author URI: http://www.wrensoft.com
License: Copyright 2014-2016  WRENSOFT  (email : zoom@wrensoft.com)
*/

/* Change Log

V0.1.0000 - Apr 30 2014
-Original Release

V0.1.1000 - Oct 29 2014
-Bug fix. Updated detection method for $_SERVER[HTTPS] variable. On IIS (using ISAPI), $_SERVER[HTTPS] is set (even connecting via http://, value is 'off') and would incorrectly form the $zoom_url. 

V0.1.2000 - Dec 17 2014
-Bug fix. Added support for array-style Zoom query string params, e.g. zoom_cat. (User contributed - J.W. of Atomic Crayon)

V0.1.3000 - Jan 9 2015
-Bug fix. The plugin will no longer hook the search if in admin section (e.g. Dashboard or Admin Panel). (User Contributed - J. A. de J.)

V0.1.4000 - Feb 26 2015
-Bug fix. When hooking the WP search, a infinite redirect can occur.

V0.1.5000 - Aug 23 2016
-Bug fix. CURL Malformed URL with WordPress 4.6.

V0.1.6000 - Sep 16 2016
-Bug fix. WordPress automatically added slashes when using double quotes, now removed with stripslashes. (User Contributed - J. A. de J.)

V0.1.7000 - Jan 24 2022
-Bug fix. "PHP Warning:  Use of undefined constant body". Changed $res[body] to $res["body"].

*/

$zoom_debug = false;

// Set activation hook
register_activation_hook( __FILE__, 'zoom_plugin_activate' );

function zoom_plugin_activate()
{
	// Create search page if:
	//  -has search page title
	//  -AND page does not already exist
	$page_title = get_option('search_page_title');
	if( $page_title ) 
	{
		$page = get_page_by_title( $page_title );
		if( $page == NULL )
		{
			$new_page = array(
				'post_title' => $page_title,
				'post_content' => '[zoomsearch]',
				'post_status' => 'publish',
				'post_type' => 'page',
				'comment_status' => 'closed'
			);
			
			$page_id = wp_insert_post( $new_page );
			update_option( 'search_page_id', $page_id );
		}
		else
		{
			$update_page = array(
				'ID' => $page->ID,
				'post_status' => 'publish'
			);
			
			wp_update_post( $update_page );
		}
	}			
}

// Set deactivation hook
register_deactivation_hook( __FILE__, 'zoom_plugin_deactivate' );

function zoom_plugin_deactivate()
{
	// Deactivate page
	$page_title = get_option( 'search_page_title' );
	if( $page_title )
	{
		$page = get_page_by_title( $page_title );
		if( $page )
		{
			$update_page = array(
				'ID' => $page->ID,
				'post_status' => 'draft'
			);
			
			wp_update_post( $update_page );
		}
	}
	
}

// Uninstall hook
register_uninstall_hook( __FILE__, 'zoom_plugin_uninstall' );

function zoom_uninstall_hook()
{
		delete_option( 'path_to_index_files' );
		delete_option( 'search_page_title' );
		delete_option( 'search_page_id' );
		delete_option( 'hook_wordpress_search' );
}

// Register Style sheet.
add_action( 'wp_enqueue_scripts', 'zoom_register_styles' );

function zoom_register_styles()
{
	wp_register_style( 'zoom-styles', plugins_url() . '/zoomsearch/zoomsearch.css' );	
}

// Add admin Menu
add_action( 'admin_menu', 'zoom_plugin_settings' );

function zoom_plugin_settings()
{
	add_options_page( 'Zoom Search Plugin Settings', 'Zoom Search Plugin Settings', 'manage_options',  __FILE__, 'zoom_display_settings' );	
	
	// call register settings function
	add_action( 'admin_init', 'zoom_register_settings' );
}

function zoom_register_settings()
{
	register_setting( 'zoom-settings-group', 'path_to_index_files' );
	register_setting( 'zoom-settings-group', 'search_page_title' );	
	register_setting( 'zoom-settings-group', 'search_page_id' );
	
	register_setting( 'zoom-settings-group', 'hook_wordpress_search' );
}

function zoom_display_settings()
{
	
	// Save the CSS
	if( isset($_POST['zoom_css']) )
	{
		$fh = fopen( plugin_dir_path(__FILE__) . '/zoomsearch.css', 'w' );
		if( $fh )
		{
			fwrite( $fh, $_POST['zoom_css'] );
			fclose( $fh );
		}
		else
		echo "failed to open file";				
	}
	else
	{
		foreach( $_POST as $key => $value )
			echo "$key";
	}		
	
	
?>
<div class="wrap">
	<?php screen_icon(); ?>
<h2>Zoom Search - WordPress Plugin Settings</h2>
<p>Easily incorporate Zoom Search, a fast custom website search engine, using pre-indexed data files into your WordPress site. To get started, please visit <a href="http://www.wrensoft.com/zoom/support" target="_blank">Setting up Zoom for use in WordPress</a>.</p>

<form method="post" action="options.php">	
<?php settings_fields( 'zoom-settings-group' ); ?>
<?php do_settings_sections( 'zoom-settings-group' ); ?>
<table class="form-table">
	<tr valign="top">
		<th scope="row">Path to index files</th>
		<td>
			<input type="text" name="path_to_index_files" value="<?php echo get_option('path_to_index_files'); ?>"/>
			<p class="description">The path should be from the server root (e.g. if your search files are located at <code>http://www.my-website.com/search/</code> directory, then the path you should enter is <code>/search</code>).</p>
		</td>		
	</tr>	
	<tr valign="top">
		<th scope="row">Search Page Title</th>
		<td>
			<input type="text" name="search_page_title" value="<?php echo get_option('search_page_title'); ?>"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
			<span>Zoom Indexer Link Back URL: <code><?php echo esc_url( get_permalink( get_page_by_title( get_option('search_page_title') ) ) ); ?></code></span>
			<p class="description">If page does not exist, it will be created. If using an existing page, you'll need to add the shortcode <code>[zoomsearch]</code> to the page's content to enable Zoom Search.</p>
		</td>
	</tr>
	<tr valign="top">
		<th scope="row">Hook WordPress Search Function</th>
		<td>
			<input type="checkbox" name="hook_wordpress_search" value="1" <?php if( get_option('hook_wordpress_search') ) echo "checked"; ?> />
			<p class="description">Zoom will override WordPress built-in text search feature if enabled.</p>
			
		</td>		
	</tr>	
</table>

<?php submit_button(); ?>
</form>


<?php 
	// CSS
	
	$css_text = "";
	$css_lines = 0;
	
	$fh = fopen( plugins_url() . '/zoomsearch/zoomsearch.css', 'r' );
	if( $fh )
	{
		while(($buffer = fgets($fh, 4096)) !== false )
		{
			$css_text .= $buffer;
			$css_lines++;
		}
		fclose( $fh );
	}		
?>

<hr>

<form method="post" action="">
<h2>Zoom Search - Styling the Search Form (CSS)</h2>
<p>You can change the fonts, colors, and styles of your search results with the CSS below.
	For some examples and more information on Cascading Style Sheets (CSS), visit our support page at:
<a href="http://www.wrensoft.com/zoom/support/css.html" target="_blank">http://www.wrensoft.com/zoom/support/css.html</a></p>
<textarea style="width:50%; margin-bottom:10px; display:block;" name="zoom_css" rows="<?php echo $css_lines; ?>"><?php echo $css_text; ?></textarea>
<input type="submit" value="Update CSS" class="button button-primary"/>
</form>

</div>
<?php

	if( isset($_GET['settings-updated']) )
		if( $_GET['settings-updated'] === 'true' )
		{
			$page_title = get_option('search_page_title');
			if( $page_title ) 
			{
				$page = get_page_by_title( $page_title );
				if( $page == NULL )
				{
					$new_page = array(
						'post_title' => $page_title,
						'post_content' => '[zoomsearch]',
						'post_status' => 'publish',
						'post_type' => 'page',
						'comment_status' => 'closed'
					);
					
					$page_id = wp_insert_post( $new_page );
					update_option( 'search_page_id', $page_id );
				}
				else
				{
					$update_page = array(
						'ID' => $page->ID,
						'post_status' => 'publish',
						//'post_content' => '[zoomsearch]'
					);
					
					wp_update_post( $update_page );
				}
			}
						
		}	
}

/**
 * Sanitize each setting field as needed
 *
 * @param array $input Contains all settings fields as array keys
 */
function zoom_santize_settings( $input )
{
	$new_input = array();
	
	return $input;
}

// Display a settings link on the main Plugins page
add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'zoom_plugin_action_links' );

function zoom_plugin_action_links( $links )
{	
	$mylinks = array( '<a href="' . admin_url( 'options-general.php?page=' . plugin_basename( __FILE__ ) ) . '">Settings</a>', );
	return array_merge( $mylinks, $links );
}

add_shortcode( 'zoomsearch', 'zoom_search' );

function zoom_search( $atts )
{
	global $zoom_debug;
	
	wp_enqueue_style('zoom-styles');
	
	// Get the location where the index files are located
	$zoom_location = get_option('path_to_index_files');
	if( !$zoom_location )
		return "<h2>Zoom Search</h2>Error: Path to Index Files not specified.";
		
	if( !file_exists( $_SERVER['DOCUMENT_ROOT'] . $zoom_location ) )
		return "<h2>Zoom Search</h2>Error: Cannot locate directory with Zoom Index files.";
				
	// Form the URL	
	$root = 'http';
	if( isset($_SERVER['HTTPS']) )
		if( !empty( $_SERVER['HTTPS'] ) )
			if( strcasecmp( $_SERVER['HTTPS'], 'off' ) != 0 )
				$root .= 's';
	$root .= '://' . $_SERVER['HTTP_HOST'];
	$zoom_url = $root . $zoom_location . '/search.php';
	if( isset( $_GET ) )
	{
		$zoom_url .= '?';
		foreach( $_GET as $key => $value )			
			if( preg_match( '/^zoom_/', $key ) ) 				// Append only valid zoom parameters
				if( is_array( $value ) )
					foreach( $value as $item )
						$zoom_url .= $key . '[]=' . urlencode(stripslashes($item)) . '&';
				else
					$zoom_url .= $key . '=' . urlencode(stripslashes($value)) . '&';
	}
	
	$zoom_text = "";
	if( false )
	{
		$ch = curl_init();
			
		curl_setopt($ch, CURLOPT_URL, $zoom_url );
		curl_setopt($ch, CURLOPT_HEADER, 0 );
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true );
		
		$zoom_text = curl_exec($ch);
				
		if( !$zoom_text )
			$zoom_text = "<h2>Zoom Search</h2>Error: " . curl_strerror( curl_errno( $ch ) );
									
		curl_close( $ch );	
		
		if( $zoom_debug ) 
			$zoom_text = "<strong>ZOOM DEBUG</strong>: CURL QUERY => '" .  $zoom_url . "'<BR>" . $zoom_text;
	}
	else
	{
		$res = wp_remote_get( $zoom_url );
		if( is_wp_error( $res ) )
			return "<h2>Zoom Search</h2>Zoom URL:$zoom_url<BR>Error: " . $res->get_error_message();
		else
		{
			if( $zoom_debug )
				$zoom_text = "<strong>ZOOM DEBUG</strong>: URL => '" .  $zoom_url . "'<BR>";
				
			$zoom_text .= $res["body"];
		}
	}
			
	return $zoom_text;
}

// Hook WordPress Search Query
add_action( 'pre_get_posts', 'zoom_hook_wp_search' );

function zoom_hook_wp_search( $query )
{
	if( get_option( 'hook_wordpress_search' ) != 1)
		return $query;
		
	if( is_admin() )
		return $query;
	
	if( $query->is_main_query() )
		if( $query->is_search )
		{				
			$page_title = get_option( "search_page_title" );
			$page = get_page_by_title( $page_title );
			
			$url = home_url( '/' ) . get_page_uri( $page->ID ); // The absolute URI, else redirect loop can occur
			$url .= '?zoom_query=' . urlencode(stripcslashes(get_search_query(false)));
			wp_redirect( $url );	
			exit();
			return $query;
		}
		
	return $query;	
}

?>